/*
 * @Borehole.java Version 1.1 10/12/2010
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

import iqstrat.iqstratStruct;

import las.gui.lasStartAppPanel;

import cmn.cmnStruct;

/** Class Borehole
 *  <p> This Class is the starting point for the Colorlith Viewer.
 *
 *  @version 1.1 10/12/2010
 *  @author  John R. Victorine
 */

public class Borehole extends Applet
{
  // Runtime object

  private Runtime r = Runtime.getRuntime();

  // Web Application is runing Stand Alone Indicator

  private boolean isStandalone = false;

  // LAS Viewer Global Variables

  private iqstratStruct stStruct = new iqstratStruct();
  private cmnStruct     stCMN    = new cmnStruct();
  private String        sPath    = new String(".");

  private String        sGEMINI  = new String("");

  // Array of LAS 3.0 File Data to read into program

  private boolean       bLAS[]   = { true,   // Version Information ( NO Data Sections)
                                     true,   // Well Information    ( NO Data Sections)
                                     true,   // Log Data Section
                                     false,  // Core Data Section
                                     false,  // Drilling Data Section
                                     false,  // Inclinometry Data Section
                                     false,  // Tops Data Section
                                     false,  // Test Data Section
                                     false,  // Perforation Data Section
                                     true,   // Control Data Section
                                     true,   // Selected Log Curves  ( NO Data Sections)
                                     false,  // Selected Core Curves ( NO Data Sections)
                                     false,  // Sequence Stratigraphy
                                     false,  // PfEFFER Flow Units
                                     false,  // PfEFFER Table Data ( READ ONLY )
                                     false,  // Geologist report Data
                                     false,  // Gamma API Shale Limits
                                     false,  // Rock Images Data
                                     false,  // Brine Data Data
                                     false,  // Bio Stratigraphy Data
                                     false}; // Depositional Environment Data

  /** getParameter()
   *  <p> This method wil get a parameter value
   *  @param key = The Name of the Parameter
   *  @param def = The Object Type of the Parameter
   *  @return Parameter
   */

  public String getParameter(String key, String def)
  {
    return (isStandalone ? System.getProperty(key, def) :
            (getParameter(key) != null ? getParameter(key) : def));
  }

  /** Constructor Borehole()
   *  <p> This is the Constructor for this class.
   */

  public Borehole() {}

  /** Method init()
   *  <p> This Method will retrieve the data values from the HTML or the main()
   *      arguments and run the Stratigraphic Data Collection and Profile
   *      Viewer Module.
   */

  public void init()
  {
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will launch the Field LAS Catalog.
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    lasStartAppPanel panel = null;

    // initialize Global LAS Data Structure

    stCMN.bStandalone   = isStandalone;
    stStruct.depthStart = 0.0;
    stStruct.depthEnd   = 0.0;

    // Retrieve KGS Image Icon for printing

    if (!isStandalone)
    {
      stCMN.urlDirCodeBase = getCodeBase();
      stCMN.appletContext  = getAppletContext();
      stCMN.pIcon          = getImage(getCodeBase(), cmn.cmnStruct.ICON);
      stCMN.pIcon2         = getImage(getCodeBase(), cmn.cmnStruct.ICON2);
    }
    else
    {
      URL url = null;

      stCMN.urlDirCodeBase =  getClass().getClassLoader().getResource(".");

      ClassLoader cl = this.getClass().getClassLoader();
      sPath = cl.getResource("images").toString();
      stCMN.sImagePath = new String(sPath);

      try
      {
        url = new URL(sPath+"\\kgsSm.png");
        stCMN.pIcon = Toolkit.getDefaultToolkit().getImage(url);
        url = new URL(sPath+"\\kgsSm2.png");
        stCMN.pIcon2 = Toolkit.getDefaultToolkit().getImage(url);
      }
      catch (java.net.MalformedURLException e)
      {
        e.printStackTrace();
        String sMessage = new String("Borehole(): \n" +
                              e.getMessage() + "\n" +
                              "Couldn't create image: " + sPath+"\\kgsSm.gif");
        JOptionPane.showMessageDialog((Component) null,
                                      sMessage,
                                      "ERROR",
                                      JOptionPane.ERROR_MESSAGE);
      }
    }

    // Get the Path of the XML Files stored in JAR File.

    ClassLoader cl = this.getClass().getClassLoader();
    sPath = cl.getResource("data").toString();
    stCMN.sPath = new String(sPath);

    sPath = cl.getResource("images").toString();
    stCMN.sImagePath = new String(sPath);
    stCMN.sGEMINI    = new String(sGEMINI);
    stStruct.bLAS    = bLAS;

    stCMN.cl = cl;

    stStruct.stCMN = stCMN;

    // Display frame

    if (!isStandalone)
    {
      panel = new lasStartAppPanel(null, stStruct);

      this.setLayout(new BorderLayout());
      this.add(panel, BorderLayout.CENTER);
    }
    else
    {
      JFrame frame = new JFrame();

      frame.setTitle("Borehole Plot");
      panel = new lasStartAppPanel(frame, stStruct);
      frame.getContentPane().add(panel, null);

      // Set the Size and Display the Frame

      frame.setSize(260, 400);

      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      frame.setLocation((d.width - frame.getSize().width)/2,
                        (d.height - frame.getSize().height)/2);

      frame.setResizable(false);
      frame.setVisible(true);
    }
  }

  /* ======================================================================= *
   * --------------------------- APPLET METHODS ---------------------------- *
   * ======================================================================= */

  /** Method start()
   *  <p> This method will start the applet
   */

  public void start() { }

  /** Method stop()
   *  <p> This method will stop the applet
   */

  public void stop() { }

  /** Method destroy()
   *  <p> This method will destroy the applet
   */

  public void destroy() { }

  /** Method getAppletInfo()
   *  <p> This method will return Applet information
   *  @return - Production Plot
   */

  public String getAppletInfo()
  {
    return ("Zonation by Depth Constraint Clustering Analysis Viewer Applet");
  }

  /** Method getParameterInfo()
   *  <p> This method will get the parameter info
   *  @return pinfo - Array of Parameters Expected.
   */

  public String[][] getParameterInfo()
  {
    String[][] pinfo =  { };

    return pinfo;
  }

  /** Method main()
   *  <p> This method will allow this Module to run either as an Application or
   *      an Applet.
   *  @param args = The Arguments passed in to query for the production data
   */

  public static void main(String[] args)
  {
    Borehole applet = new Borehole();
    applet.isStandalone = true;
    applet.init();
    applet.start();
  }
}