/*
 * @lasStartFrame.java Version 1.1 06/09/2014
 *
 * Copyright (c) 2014 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package las.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import cmn.cmnStruct;			            // Global Common Data Structure
import cmn.cmnLASFileListStruct;            // LAS File List Data Structure
import cmn.cmnLASFileStruct;                // LAS File Data Structure

import iqstrat.iqstratStruct;               // Global IQSTRAT Data Structure
import iqstrat.iqstratControlStruct;        // Plot Control Data Structure
import iqstrat.iqstratHeadersStruct;        // Headers Data Structure

import kgs.gui.kgsSearchKGSFrame;           // KGS Search KGS Well Data Dialog

import las.lasFileDataStruct;               // LAS File Data Structure
import las.gui.lasPlotFrame;                // LAS Plot Frame

import las3.las3ListStruct;                 // LAS 3.0 List Data Structure
import las3.las3Struct;                     // LAS 3.0 Data Structure
import las3.io.las3Read;                    // Read LAS 3.0 File

/** Class lasStartFrame
 *  <p> This Class Starting Panel for this web application
 *
 *  @version 1.1 06/09/2014
 *  @author  John R. Victorine
 */

public class lasStartFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private iqstratStruct  stStruct = null;  // Global IQSTRAT Data Structure

  // Global Variables

  private Observable     notifier = null;  // Observables Object

  private int            iAction  = -1;   // The type of search indicator
  private int            iWait    = 0;

  // -- Load Profile Data Frames

  public static final int _KGS          = 1;
  public static final int _LAS_3_FILE   = 4;

  private kgsSearchKGSFrame    pKGS       = null; // Search KGS Database for Data
  private las3Read             p3         = null; // Read LAS 3 File for Data

  // -- Type of data

  public static final int _HEADERS     = 0;  // Headers Data

  // -- LAS File & Geological Data

  public static final int _LAS3        = 1;  // LAS File Data
  public static final int _LAS         = 2;  // LAS File Data

  // .. Plot Frame

  private lasPlotFrame         plot    = null;

  // .. Plot Control

  private iqstratControlStruct stControl  = new iqstratControlStruct();

  // .. Header Information Data Structure

  private iqstratHeadersStruct stHeader   = null; // Header Data Structure

  // .. LAS Data

  private int                  MAX_LOG    = 3;
  private int                  iLog       = 0;
  private lasFileDataStruct    stLog[]    = null; // LAS File Data Struct
  private cmnLASFileListStruct stFiles    = null; // LAS File Info List Structure

  private lasFileDataStruct    stLAS      = null; // LAS File Data Struct

  // Global Panel Widgets

  // -- Top Panel Widgets

  private JButton   btnKGS1 = null;  // KGS Well Data
  private JButton   btnPC1  = null;  // Log ASCII Stadard File

  private JLabel lblLASFile1 = new JLabel();
  private JLabel lblLASFile2 = new JLabel();
  private JLabel lblLASFile3 = new JLabel();

  // -- LAS Filename Widgets

  private JTextField txtLAS1   = new JTextField(); // 1st LAS File
  private JTextField txtLAS2   = new JTextField(); // 2nd LAS File
  private JTextField txtLAS3   = new JTextField(); // 3rd LAS File

  // -- Data Loaded Panel Widgets

  private static final int _LABEL   = 0;  // Table Header
  private static final int _LABEL2  = 4;  // Table Header
  private static final int _LAS_R   = 1;  // Log Data Types

  private static final int _NO_C    = 1;  // NO Data Column
  private static final int _LAS_3C  = 2;  // LAS File
  private static final int _LAS_C   = 3;  // LAS File
  private static final int _CSV_C   = 4;  // CSV File
  private static final int _KGS_C   = 5;  // KGS Database

  private static final String EMPTY = "....";
  private static final String NO    = " NO ";
  private static final String YES   = "YES ";

  private static final int _TOTAL = 2;
  private static final int _COLS  = 6;

  private static final int _TABLE_ROW = 2;
  private String ARRAY_TEXT[][] = {
    { "Data Type               ",  "    ", "3.0 ", "LAS ", "CSV ", "KGS " },
    { "Log Data                ",  " NO ", "....", "....", "....", "...." },
  };

  private JLabel lblWait  = new JLabel();

  private JLabel lbl[][]  = null;

  // Type of data loaded

  private static final int _CALIPERS = 12;

  private int        iAzimuth   = 0;
  private int        iCalipers  = 0;
  private double     degrees    = 0.0;

  private JTextField txtCAL = new JTextField(); // Number of Calipers
  private JTextField txtAng = new JTextField(); // Angle between Calipers

  private JCheckBox  cb[]   = null;             // At least one Azimuth & one Caliper

  // -- Base Panel Widgets

  private JButton btnContinue  = new JButton();
  private JButton btnClear     = new JButton();
  private JButton btnExit      = new JButton();

  /** Constructor lasStartFrame( iqstratStruct stStruct )
   *  <p> This is the Constructor for this class.
   *  @param stStruct = Global IQSTRAT Data Structure
   */

  public lasStartFrame( iqstratStruct stStruct )
  {
    try
    {
      this.stStruct = stStruct;

      stFiles        = new cmnLASFileListStruct();
      stFiles.stItem = new cmnLASFileStruct[MAX_LOG];
      stLog          = new lasFileDataStruct[MAX_LOG];

      for (int i=0; i<MAX_LOG; i++)
      {
        stFiles.stItem[i] = new cmnLASFileStruct();
        stLog[i]          = new lasFileDataStruct();
	  }

      jbInit();

      addWindowListener( new lasStartFrame_WindowListener() );
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int icnt = 0;

    JPanel pnlTop         = new JPanel();
    JPanel pnlKGS         = new JPanel();
    JPanel pnlKGSCenter   = new JPanel();
    JPanel pnlKGSicon1    = new JPanel();
    JPanel pnlPC          = new JPanel();
    JPanel pnlPCCenter    = new JPanel();
    JPanel pnlPCicon1     = new JPanel();

    JPanel pnlCenter      = new JPanel();
    JPanel pnlCenterTop   = new JPanel();
    JPanel pnlCenterTop1  = new JPanel();

    JPanel pnlTable       = new JPanel();
    JPanel pnlTable1      = new JPanel();
    JPanel pnlTable2      = new JPanel();

    JPanel pnlLAS         = new JPanel();
    JPanel pnlLASCenter   = new JPanel();
    JPanel pnlLASFile1    = new JPanel();
    JPanel pnlLASFile2    = new JPanel();
    JPanel pnlLASFile3    = new JPanel();

    JLabel lblLAS         = new JLabel();

    JPanel pnl[]          = null;
    JPanel pnlE[]         = null;
    JPanel pnlC[]         = null;

    JPanel pnlCALCurves   = new JPanel();
    JPanel pnlNoCalipers  = new JPanel();
    JPanel pnlAngSep      = new JPanel();

    JPanel pnlCalipers    = new JPanel();
    JPanel pnlAzimuth     = new JPanel();
    JPanel pnlAzimuth1    = new JPanel();
    JPanel pnlCaliper     = new JPanel();
    JPanel pnlCaliper1    = new JPanel();

    JLabel lblAzimuth     = new JLabel();
    JLabel lblCaliper     = new JLabel();

    JPanel pnlBase        = new JPanel();
    JPanel pnlWait        = new JPanel();
    JPanel pnlBase1       = new JPanel();
    JPanel pnlBase2       = new JPanel();

    TitledBorder titledBorderTop = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Data Source");
    titledBorderTop.setTitleFont(new java.awt.Font("Dialog", 1, 14));

    TitledBorder titledBorderKGS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "KGS (Database & Server)");
    titledBorderKGS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderKGS1 = new TitledBorder(
        BorderFactory.createEmptyBorder(), "Well Data");
    titledBorderKGS1.setTitleJustification(TitledBorder.CENTER);
    titledBorderKGS1.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderPC = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "PC (ASCII Data Files)");
    titledBorderPC.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderPC1 = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "Ver 2.0 & 3.0");
    titledBorderPC1.setTitleJustification(TitledBorder.CENTER);
    titledBorderPC1.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderCenter = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Data Loaded");
    titledBorderCenter.setTitleFont(new java.awt.Font("Dialog", 1, 14));

    TitledBorder titledBorderFile = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Data Source Filenames:");
    titledBorderFile.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderLAS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Log ASCII Standard (LAS) Files:");
    titledBorderLAS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderCalipers = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Calipers");
    titledBorderCalipers.setTitleFont(new java.awt.Font("Dialog", 1, 14));

    TitledBorder titledBorderCAL = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Number of Calipers");
    titledBorderCAL.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderAng = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Angle Between [degs]");
    titledBorderAng.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Create an Observable

    notifier = new lasStartFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.setTitle( "Load Data" );
    this.getContentPane().setLayout( new BorderLayout() );
    this.getContentPane().setBackground(Color.white);

    // Top Panel

    pnlTop.setBorder( titledBorderTop );
    pnlTop.setLayout( new GridLayout() );
    pnlTop.setBackground(Color.white);

    // -- KGS Data Source

    pnlKGS.setBackground(Color.white);
    pnlKGS.setLayout( new BorderLayout() );

    // .. KGS Center Panel

    pnlKGSCenter.setBackground(Color.white);
    pnlKGSCenter.setBorder( titledBorderKGS );
    pnlKGSCenter.setLayout( new GridLayout() );

    // .... KGS - LAS Search

    pnlKGSicon1.setBackground(Color.white);
    pnlKGSicon1.setBorder( titledBorderKGS1 );

    btnKGS1 = new JButton( createImageIcon( cmn.cmnStruct.WELL_DATA ) );
    btnKGS1.setBackground(Color.white);
    btnKGS1.setPreferredSize( new Dimension(64, 64) );
    btnKGS1.addActionListener( this );

    // -- PC Data Source

    pnlPC.setBackground(Color.white);
    pnlPC.setLayout( new BorderLayout() );

    // .. PC Center Panel

    pnlPCCenter.setBackground(Color.white);
    pnlPCCenter.setBorder( titledBorderPC );
    pnlPCCenter.setLayout( new GridLayout() );

    // .. .. Log ASCII Standard version 2.0 or 3.0 Files Button Panel

    pnlPCicon1.setBackground(Color.white);
    pnlPCicon1.setBorder( titledBorderPC1 );

    btnPC1 = new JButton( createImageIcon( cmn.cmnStruct.LAS_FILE ) );
    btnPC1.setBackground(Color.white);
    btnPC1.setPreferredSize( new Dimension(64, 64) );
    btnPC1.addActionListener( this );

    // Center Panel

    pnlCenter.setBorder( titledBorderCenter );
    pnlCenter.setLayout( new BorderLayout() );
    pnlCenter.setBackground(Color.white);

    // -- Center Top Panel

    pnlCenterTop.setBorder( titledBorderFile );
    pnlCenterTop.setBackground(Color.white);
    pnlCenterTop.setLayout( new GridLayout() );

    // .. LAS Filenames

    pnlCenterTop1.setLayout( new BorderLayout() );
    pnlCenterTop1.setBackground(Color.white);

    // .. .. LAS Label Panel

    pnlLAS.setBackground(Color.white);

    lblLAS.setHorizontalAlignment(SwingConstants.LEFT);
    lblLAS.setFont(new java.awt.Font("Monospaced", 1, 11));
    lblLAS.setBackground(Color.white);
    lblLAS.setText("LAS:   ");

    pnlLASCenter.setBorder(titledBorderLAS);
    pnlLASCenter.setLayout( new GridLayout(3,1) );
    pnlLASCenter.setBackground(Color.white);

    // .. .. .. LAS File 1 Panel

    pnlLASFile1.setLayout( new BorderLayout() );
    pnlLASFile1.setBackground(Color.white);

    lblLASFile1.setHorizontalAlignment(SwingConstants.LEFT);
    lblLASFile1.setFont(new java.awt.Font("Monospaced", 1, 11));
    lblLASFile1.setBackground(Color.white);
    lblLASFile1.setText("1: ");

    txtLAS1.setFont(new java.awt.Font("Dialog", 1, 11));
    txtLAS1.setEditable(false);
    txtLAS1.setForeground(Color.blue);
    txtLAS1.setBackground(Color.white);
    txtLAS1.setText("");

    // .. .. .. LAS File 2 Panel

    pnlLASFile2.setLayout( new BorderLayout() );
    pnlLASFile2.setBackground(Color.white);

    lblLASFile2.setHorizontalAlignment(SwingConstants.LEFT);
    lblLASFile2.setFont(new java.awt.Font("Monospaced", 1, 11));
    lblLASFile2.setBackground(Color.white);
    lblLASFile2.setText("2: ");

    txtLAS2.setFont(new java.awt.Font("Dialog", 1, 11));
    txtLAS2.setEditable(false);
    txtLAS2.setForeground(Color.blue);
    txtLAS2.setBackground(Color.white);
    txtLAS2.setText("");

    // .. .. .. LAS File 3 Panel

    pnlLASFile3.setLayout( new BorderLayout() );
    pnlLASFile3.setBackground(Color.white);

    lblLASFile3.setHorizontalAlignment(SwingConstants.LEFT);
    lblLASFile3.setFont(new java.awt.Font("Monospaced", 1, 11));
    lblLASFile3.setBackground(Color.white);
    lblLASFile3.setText("3: ");

    txtLAS3.setFont(new java.awt.Font("Dialog", 1, 11));
    txtLAS3.setEditable(false);
    txtLAS3.setForeground(Color.blue);
    txtLAS3.setBackground(Color.white);
    txtLAS3.setText("");

    // -- Center Table Panel

    pnlTable.setLayout( new GridLayout() );
    pnlTable.setBackground(Color.white);

    pnlTable1.setLayout( new GridLayout(2,1) );
    pnlTable1.setBackground(Color.white);
//    pnlTable1.setPreferredSize( new Dimension(10, 30) );

    pnl  = new JPanel[_TOTAL];
    pnlE = new JPanel[_TOTAL];
    pnlC = new JPanel[_TOTAL];
    lbl  = new JLabel[_TOTAL][6];

    for (int i= 0; i<_TOTAL; i++)
    {
	  pnl[i] = new JPanel();
      pnl[i].setLayout( new BorderLayout() );
      pnl[i].setBackground(Color.white);

	  pnlE[i] = new JPanel();
      pnlE[i].setBackground(Color.white);

	  pnlC[i] = new JPanel();
      pnlC[i].setLayout( new GridLayout() );
      pnlC[i].setBackground(Color.white);

      for (int j=0; j<_COLS; j++)
      {
		lbl[i][j] = new JLabel();
        lbl[i][j].setHorizontalAlignment(SwingConstants.LEFT);
        lbl[i][j].setFont(new java.awt.Font("Monospaced", 1, 11));
        lbl[i][j].setBackground(Color.white);
        lbl[i][j].setText( ARRAY_TEXT[i][j] );

        if (j>0)
        {
          pnlC[i].add( lbl[i][j], null );
		}
		else
		{
          pnlE[i].add( lbl[i][0], null );
		}
      }

      pnl[i].add( pnlE[i],    BorderLayout.WEST );
      pnl[i].add( pnlC[i],    BorderLayout.CENTER );

      if (i<_TABLE_ROW)
      {
        pnlTable1.add( pnl[i],  null );
      }
	}

    // Type of data loaded

    pnlTable2.setLayout( new BorderLayout() );
    pnlTable2.setBorder(titledBorderCalipers);
    pnlTable2.setBackground(Color.white);

    // -- Caliper Text Field Panel

    pnlCALCurves.setLayout( new GridLayout() );
    pnlCALCurves.setBackground(Color.white);

    pnlNoCalipers.setLayout( new BorderLayout() );
    pnlNoCalipers.setBorder( titledBorderCAL );
    pnlNoCalipers.setBackground(Color.white);

    txtCAL.setFont(new java.awt.Font("Dialog", 1, 11));
    txtCAL.setEditable(false);
    txtCAL.setForeground(Color.blue);
    txtCAL.setBackground(Color.white);
    txtCAL.setText("" + iCalipers);

    pnlCALCurves.add( pnlNoCalipers, null);
    pnlNoCalipers.add( txtCAL,       BorderLayout.CENTER );

    pnlAngSep.setLayout( new BorderLayout() );
    pnlAngSep.setBorder( titledBorderAng );
    pnlAngSep.setBackground(Color.white);

    txtAng.setFont(new java.awt.Font("Dialog", 1, 11));
    txtAng.setForeground(Color.blue);
    txtAng.setBackground(Color.white);
    txtAng.setText("" + degrees);

    pnlCALCurves.add( pnlAngSep,     null);
    pnlAngSep.add( txtAng,           BorderLayout.CENTER );

    // .. Attach the text widgets to table 2

    pnlTable2.add( pnlCALCurves,     BorderLayout.NORTH );

    // -- Calipers Panel

    pnlCalipers.setLayout( new GridLayout(2,1) );
    pnlCalipers.setBackground(Color.white);

    // .. Azimuth Panel

    pnlAzimuth.setLayout( new BorderLayout() );
    pnlAzimuth.setBackground(Color.white);

    lblAzimuth.setHorizontalAlignment(SwingConstants.LEFT);
    lblAzimuth.setFont(new java.awt.Font("Monospaced", 1, 11));
    lblAzimuth.setBackground(Color.white);
    lblAzimuth.setText("Azimuth: ");

    pnlAzimuth.add( lblAzimuth,  BorderLayout.WEST );

    pnlAzimuth1.setLayout( new GridLayout() );
    pnlAzimuth1.setBackground(Color.white);

    // .. Caliper Panel

    pnlCaliper.setLayout( new BorderLayout() );
    pnlCaliper.setBackground(Color.white);

    lblCaliper.setHorizontalAlignment(SwingConstants.LEFT);
    lblCaliper.setFont(new java.awt.Font("Monospaced", 1, 11));
    lblCaliper.setBackground(Color.white);
    lblCaliper.setText("Radius:  ");

    pnlCaliper.add( lblCaliper,  BorderLayout.WEST );

    pnlCaliper1.setLayout( new GridLayout() );
    pnlCaliper1.setBackground(Color.white);

    // .. Check boxes to turn curves on or off

    cb = new JCheckBox[_CALIPERS];

    icnt = 1;

    for (int i=0; i<_CALIPERS; i++)
    {
	  if (i == _CALIPERS/2)
	    icnt = 1;

 	  cb[i] = new JCheckBox();
 	  cb[i].setFont(new java.awt.Font("Dialog", 1, 11));
 	  cb[i].setEnabled(false);
 	  cb[i].setBackground(Color.white);
 	  cb[i].setText( "" + icnt );
 	  cb[i].addActionListener(this);

	  if (i < _CALIPERS/2)
	    pnlAzimuth1.add(cb[i], null);
	  else
	    pnlCaliper1.add(cb[i], null);

 	  icnt++;
    }

    pnlAzimuth.add( pnlAzimuth1,  BorderLayout.CENTER );
    pnlCaliper.add( pnlCaliper1,  BorderLayout.CENTER );

    pnlCalipers.add( pnlAzimuth,  null );
    pnlCalipers.add( pnlCaliper,  null );

    pnlTable2.add( pnlCalipers,   BorderLayout.SOUTH );

    // Base Panel

    pnlBase.setLayout( new BorderLayout() );
    pnlBase.setBackground(Color.white);

    pnlWait.setBackground(Color.white);

    lblWait.setHorizontalAlignment(SwingConstants.LEFT);
    lblWait.setFont(new java.awt.Font("Dialog", 1, 14));
    lblWait.setBackground(Color.white);
    lblWait.setForeground(Color.red);
    lblWait.setText("");

    pnlBase2.setBackground(Color.white);

    btnContinue.setFont(new java.awt.Font("Dialog", 1, 11));
    btnContinue.setPreferredSize( new Dimension(120, 25) );
    btnContinue.setText("Continue");
    btnContinue.addActionListener(this);

    btnClear.setFont(new java.awt.Font("Dialog", 1, 11));
    btnClear.setPreferredSize( new Dimension(120, 25) );
    btnClear.setText("Clear");
    btnClear.addActionListener(this);

    btnExit.setFont(new java.awt.Font("Dialog", 1, 11));
    btnExit.setPreferredSize( new Dimension(120, 25) );
    btnExit.setText("Exit");
    btnExit.addActionListener(this);

    // Add Widgets to Panel

    this.getContentPane().add( pnlTop,  BorderLayout.NORTH );
    pnlTop.add( pnlKGS,                 null );
    pnlKGS.add( pnlKGSCenter,           BorderLayout.CENTER );
    pnlKGSCenter.add( pnlKGSicon1,      null );
    pnlKGSicon1.add( btnKGS1,           null );

    pnlTop.add( pnlPC,                  null );
    pnlPC.add( pnlPCCenter,             BorderLayout.CENTER );
    pnlPCCenter.add( pnlPCicon1,        null );
    pnlPCicon1.add( btnPC1,             null );

    this.getContentPane().add( pnlCenter,  BorderLayout.CENTER );
    pnlCenter.add( pnlCenterTop,        BorderLayout.NORTH );

    pnlCenterTop.add( pnlCenterTop1,    null );
    pnlCenterTop1.add( pnlLASCenter,    BorderLayout.CENTER );

    pnlLASCenter.add( pnlLASFile1,      null );
    pnlLASFile1.add(  lblLASFile1,      BorderLayout.WEST );
    pnlLASFile1.add(  txtLAS1,          BorderLayout.CENTER );

    pnlLASCenter.add( pnlLASFile2,      null );
    pnlLASFile2.add(  lblLASFile2,      BorderLayout.WEST );
    pnlLASFile2.add(  txtLAS2,          BorderLayout.CENTER );

    pnlLASCenter.add( pnlLASFile3,      null );
    pnlLASFile3.add(  lblLASFile3,      BorderLayout.WEST );
    pnlLASFile3.add(  txtLAS3,          BorderLayout.CENTER );

    pnlCenter.add( pnlTable,            BorderLayout.SOUTH );
    pnlTable.add( pnlTable1,            BorderLayout.CENTER );
    pnlTable.add( pnlTable2,            BorderLayout.CENTER );

    this.getContentPane().add( pnlBase, BorderLayout.SOUTH );
//    panel.add( pnlBase,                 BorderLayout.SOUTH );
    pnlBase.add( pnlWait,               BorderLayout.NORTH );
    pnlWait.add( lblWait,               BorderLayout.CENTER );

    pnlBase.add( pnlBase2,              BorderLayout.SOUTH );
    pnlBase2.add( btnContinue,          null );
    pnlBase2.add( btnClear,             null );
    pnlBase2.add( btnExit,              null );

    setButtons();

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int iScreenHigh  = d.height-50;
//    if (iScreenHigh < 675)
    this.setSize(new Dimension(700, 500));
//    else
//      this.setSize(new Dimension(700, 675));
    this.setLocation(10, (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * ------------------------- CLOSE METHODS ----------------------------- *
   * ===================================================================== */

  /** METHOD lasStartFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class lasStartFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e) { close(); }
  }

  /** Method close()
   * <p> This method will close this dialog and all other dialogs opened by
   *     this dialog
   */

  public void close()
  {
    // Input Variables

	stStruct     = null;

    // Global Variables

    notifier     = null; // Observable notifier

    if (pKGS != null)
      pKGS.close();
    pKGS = null;

    if (p3 != null)
      p3.delete();
    p3 = null;

    if (plot != null)
      plot.close();
    plot = null;

    if (stControl != null)
      stControl.delete();
    stControl = null;

    if (stHeader != null)
      stHeader.delete();
    stHeader = null;

    // .. LAS Data

	if (iLog > 0)
	{
	  for (int i=0; i<iLog; i++)
	  {
		if (stLog[i] != null)
		  stLog[i].delete();
	    stLog[i] = null;
	  }

	  iLog = 0;
	}

    if (stFiles != null)
      stFiles.delete();
    stFiles = null;

    if (stLAS != null)
      stLAS.delete();
    stLAS  = null;        // LAS File Data Structure

    // Global Panel Widgets

    // -- Top Panel Widgets

    btnKGS1      = null;  // KGS Well Data
    btnPC1       = null;  // Log ASCII Stadard File

    lblLASFile1  = null;
    lblLASFile2  = null;
    lblLASFile3  = null;

    // -- LAS Filename Widgets

    txtLAS1      = null; // 1st LAS File
    txtLAS2      = null; // 2nd LAS File
    txtLAS3      = null; // 3rd LAS File

    ARRAY_TEXT   = null;

    lblWait      = null;;
    lbl          = null;

    // Type of data loaded

    txtCAL       = null; // Number of Calipers
    txtAng       = null; // Angle between Calipers

    cb           = null;  // At least one Azimuth & one Caliper

    // -- Base Panel Widgets

    btnContinue  = null;;
    btnClear     = null;;
    btnExit      = null;;

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getLAS()
   * <p> These methods will return the LAS File Data Structure
   * @return stLAS = LAS File Data Structure
   */

  public lasFileDataStruct getLAS()
  {
	int    i,j;
	int    iRows  = 0;
	double dStart = 0.0;
	double dEnd   = 0.0;
	double dStep  = 0.0;
	double dNull  = -999.25;
	int    iTemp  = 0;
	double dStp   = 0.0;
    las3ListStruct st3 = null;

	if (iLog > 0)
	{
      if (iLog == 1)
      {
 	    stLAS = las.lasFileDataUtility.copy( stLog[0] );
  	  }
  	  else
  	  {
        stLAS = new lasFileDataStruct();

	    for (i=0; i<iLog; i++)
	    {
          dStep  = stLog[i].depthStep;
          if (dStep > stLog[i].depthStep)
          {
            dStep  = stLog[i].depthStep;
		  }

		  if (stLog[i].iLogs > 0)
		  {
		    for (j=0; j<stLog[i].iLogs; j++)
		      st3 = las3.las3Utility.add( stLog[i].stLAS3[j], st3 );
	      }
        }

	    for (i=0; i<iLog; i++)
	    {
		  if (i==0)
		  {
		    dStart = stLog[i].depthStart;
		    dEnd   = stLog[i].depthEnd;
		    dStep  = stLog[i].depthStep;
		    dNull  = stLog[i].dNull;

            stLAS = las.lasFileDataUtility.copyHeader( stLog[i], stLAS );
		  }

		  if (dStart > stLog[i].depthStart)
		    dStart = stLog[i].depthStart;

          if (dEnd < stLog[i].depthEnd)
		    dEnd   = stLog[i].depthEnd;

          if (dStep > stLog[i].depthStep)
          {
            dStep  = stLog[i].depthStep;
		  }
	    }

        iTemp  = (int) dStart;
        dStart = (double) iTemp;

        iTemp  = 1 + (int) dEnd;
        dEnd   = (double) iTemp;

        dStp = 0.5;
        if (dStep < 0.5)
          dStp = 0.25;
        if (dStep < 0.25)
          dStp = 0.1;

        dStep = dStp;
	    iRows = (int) ( Math.abs(dEnd - dStart) / dStep ) + 1;

        stLAS.depthStart = dStart;
        stLAS.depthEnd   = dEnd;
        stLAS.depthStep  = dStep;

        stLAS = las3.las3LoadLogData.initData( iRows, stLAS );

	    for (i=0; i<iLog; i++)
	    {
          stLAS = las3.las3LoadLogData.mergeLogData( stLog[i], stLAS );
	    }
	  }
    }

//    if (stLAS != null)
//      stLAS.stList = iqstrat.iqstratShaleUtility.copyList( stShale );

    if (st3 != null)
    {
	  if (st3.iCount > 0)
	  {
	    if (stLAS != null)
	    {
		  stLAS.stLAS3 = new las3Struct[st3.iCount];
		  stLAS.iLogs  = st3.iCount;
		  for (i=0; i<st3.iCount; i++)
		  {
		    stLAS.stLAS3[i] = new las3Struct();
		    stLAS.stLAS3[i] = las3.las3Utility.copy( st3.stItem[i] );
		  }
		}
	  }
	}

    return (stLAS);
  }

  /** Method getData()
   * <p> This method will retrieve data
   * @param iLocation = Location of Data being retrieved
   * @param iData     = Data Type being retrieved
   */

  private void getData( int iLocation, int iData)
  {
	int i=0;
    double diff   = 0.0;
    double depth1 = 0.0;
    double depth2 = 0.0;

    switch (iData)
    {
      case _HEADERS:    // Headers Data
        stHeader = iqstrat.iqstratHeadersUtility.copy( getHeader( iLocation, pKGS, p3 ));
//            getHeader( iLocation, pKGS, pNotesFile, p3 ));
        break;

      case _LAS:   // LAS File Data
        if (stFiles == null)
        {
          stFiles        = new cmnLASFileListStruct();
          stFiles.stItem = new cmnLASFileStruct[MAX_LOG];
          stLog          = new lasFileDataStruct[MAX_LOG];

          for (i=0; i<MAX_LOG; i++)
          {
            stFiles.stItem[i] = new cmnLASFileStruct();
            stLog[i]          = new lasFileDataStruct();
          }
		}

        if (iLog < MAX_LOG)
        {
          stLog[iLog] = las.lasFileDataUtility.copy( getLAS( iLocation, pKGS, p3 ));
//                            iLocation,  pKGS, pNotesFile, p3 ));

          setCaliperTable( stLog[iLog] );

          stFiles.stItem[iLog].iType = util.utilFileIO.DIRECTORY;
          if (stLog[iLog].sReadMethod.equals("URL"))
            stFiles.stItem[iLog].iType = util.utilFileIO.URL_FILE;

          stFiles.stItem[iLog].sDirectory = new String( stLog[iLog].sDirectory );
          stFiles.stItem[iLog].sFilename  = new String( stLog[iLog].sFilename );
          stFiles.stItem[iLog].sVersion   = new String( stLog[iLog].sVersion );

          stFiles.stItem[iLog].dNull      = stLog[iLog].dNull;
          stFiles.stItem[iLog].depthStart = stLog[iLog].depthStart;
          stFiles.stItem[iLog].depthEnd   = stLog[iLog].depthEnd;
          stFiles.stItem[iLog].depthStep  = stLog[iLog].depthStep;

//las.lasFileDataUtility.print(stLog[iLog]);
          iLog++;
          stFiles.iCount = iLog;
	    }
        break;
    }
  }

  /** Method getHeader()
   * <p> This method will retrieve the Header Data Structure
   * @param  iLocation  = Location identifier
   * @param  pKGS       = Search KGS Database for Data
   * @param  p3         = Read LAS 3 File for Data
   * @return st         = the Header Data Structure
   */

  public static iqstratHeadersStruct getHeader(
      int                  iLocation,  // Location identifier
      kgsSearchKGSFrame    pKGS ,      // Search KGS Database for Data
      las3Read             p3 )        // Read LAS 3 File for Data
  {
    iqstratHeadersStruct st = null;

    switch (iLocation)
    {
      case _KGS:
        st = pKGS.getWellHeader();
        break;
      case _LAS_3_FILE:
        st = p3.getWellData();
        break;
    }

    return (st);
  }

  /** Method getLAS()
   * <p> This method will retrieve the LAS File Data Structure
   * @param  iLocation  = Location identifier
   * @param  pKGS       = Search KGS Database for Data
   * @param  pNotesFile = Read Remarks CSV File
   * @param  p3         = Read LAS 3 File for Data
   * @return st         = LAS File Data Structure
   */

  public static lasFileDataStruct getLAS(
      int                  iLocation,  // Location identifier
      kgsSearchKGSFrame    pKGS ,      // Search KGS Database for Data
      las3Read             p3 )        // Read LAS 3 File for Data
  {
    lasFileDataStruct st = null;

    switch (iLocation)
    {
      case _KGS:
        st = las.lasFileDataUtility.copy( pKGS.getLASData() );
        if (st != null) { st.iSource = las3.las3Constants._SOURCE_KGS; }
        pKGS.closeLASFile();
        break;
      case _LAS_3_FILE:
        st = p3.getLASData();
        if (st != null) { st.iSource = las3.las3Constants._SOURCE_PC; }
        break;
    }

    return (st);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will set the Profile Plot buttons depending on the
   *      selected data.
   */

  private void setButtons()
  {
	int iContinue = 0;

    iAction = cmn.cmnStruct._LAS;
    btnContinue.setEnabled(false);
/*
    for(int i=0; i<_TOTAL; i++)
    {
      if ((i != _LABEL) && (i != _LABEL2))
	  {
	    if (ARRAY_TEXT[i][1].equals(EMPTY))
	    {
	      iContinue = 1;
	    }
	  }
	}
*/
    if ((iAzimuth > 0) && (iCalipers > 0))
      iContinue = 1;

	if (iContinue == 1)
	{
      btnContinue.setEnabled(true);
	}
	else if ((iWait == 0) && (iContinue == 0))
	{
	  lblWait.setText( "Please select an ICON above to enter Data." );
	}

    if ((iWait == 0) && (iContinue == 1))
	{
	  lblWait.setText( "" );
	}
  }

  /** Method setDepthScale()
   *  <p> This method will set the depth range & scale depending
   *      on the data loaded.
   */

  private void setDepthScale()
  {
	double diff = 0.0;

    if (stControl == null)
      stControl = new iqstratControlStruct();

	if (stLAS != null)
	{
      stControl.depthStart = stLAS.depthStart;
      stControl.depthEnd   = stLAS.depthEnd;
	}

    diff = stControl.depthEnd - stControl.depthStart;
    if (diff < 10)
      stControl.iScale = iqstrat.iqstratTracksStruct.SCALE_1;
    else if (diff < 20)
      stControl.iScale = iqstrat.iqstratTracksStruct.SCALE_2;
    else if (diff < 50)
      stControl.iScale = iqstrat.iqstratTracksStruct.SCALE_5;
    else if (diff < 100)
      stControl.iScale = iqstrat.iqstratTracksStruct.SCALE_10;
    else if (diff < 200)
      stControl.iScale = iqstrat.iqstratTracksStruct.SCALE_20;
    else if (diff < 500)
      stControl.iScale = iqstrat.iqstratTracksStruct.SCALE_50;
    else
      stControl.iScale = iqstrat.iqstratTracksStruct.SCALE_100;
  }

  /** Method setTable()
   *  <p> This will set the data settings
   */

  private void setTable()
  {
    for (int i=0; i<_TOTAL; i++)
    {
	  for (int j=1; j<_COLS; j++)
	  {
		if ((i!=0) && (i!=_TABLE_ROW))
	    lbl[i][j].setText( ARRAY_TEXT[i][j] );
	    if (ARRAY_TEXT[i][j].equals(YES))
	    {
		  lbl[i][j].setForeground(Color.blue);
		}
	  }
	}
  }

  /** Method setCaliperTable
   *  <p> This method will set the Caliper Table values
   *  @param st = the las file data structure
   */

  private void setCaliperTable(lasFileDataStruct st)
  {
	if (st != null)
	{
	  if (st.checkData(las.lasStandardTools._AZIC1))
	  {
		cb[0].setSelected(true);
		iAzimuth++;
	  }

	  if (st.checkData(las.lasStandardTools._CAL1))
	  {
		cb[6].setSelected(true);
		iCalipers++;
	  }

	  if (st.checkData(las.lasStandardTools._CAL2))
	  {
		cb[7].setSelected(true);
		iCalipers++;
	  }

	  if (st.checkData(las.lasStandardTools._CAL3))
	  {
		cb[8].setSelected(true);
		iCalipers++;
	  }

	  if (st.checkData(las.lasStandardTools._CAL4))
	  {
		cb[9].setSelected(true);
		iCalipers++;
	  }

	  if (st.checkData(las.lasStandardTools._CAL5))
	  {
		cb[10].setSelected(true);
		iCalipers++;
	  }

	  if (st.checkData(las.lasStandardTools._CAL6))
	  {
		cb[11].setSelected(true);
		iCalipers++;
	  }

	  if (iCalipers > 0)
	  {
	    degrees = 360.0 / iCalipers;
	    txtCAL.setText( "" + iCalipers );
	    txtAng.setText( "" + degrees );
	  }
	}

	setButtons();
  }

  /** Method clear()
   * <p> This method will clear the data structures
   */

  private void clear()
  {

    stFiles        = new cmnLASFileListStruct();
    stFiles.stItem = new cmnLASFileStruct[MAX_LOG];
    stLog          = new lasFileDataStruct[MAX_LOG];

    for (int i=0; i<MAX_LOG; i++)
    {
      stFiles.stItem[i] = new cmnLASFileStruct();
      stLog[i]          = new lasFileDataStruct();
    }

  	txtLAS1.setText( "" );   // LAS File 1
   	txtLAS2.setText( "" );   // LAS File 2
   	txtLAS3.setText( "" );   // LAS File 3

    for (int i=0; i<_TOTAL; i++)
    {
	  for (int j=1; j<_COLS; j++)
	  {
		if ((i!=0) && (i!=_TABLE_ROW))
		{
	      if (j == _NO_C)
	      {
			ARRAY_TEXT[i][_NO_C] = new String( NO );
		  }
	      else
	      {
		    ARRAY_TEXT[i][j] = new String( EMPTY );
		  }
	    }
	  }
	}

    iAzimuth  = 0;
    iCalipers = 0;
    degrees   = 0.0;

    txtCAL.setText( "" + iCalipers );
    txtAng.setText( "" + degrees );

	for (int i=0; i<_CALIPERS; i++)
	{
	  cb[i].setSelected(false);
	}

    iAction = cmn.cmnStruct._LAS;
	setTable();
  }

  /* ===================================================================== *
   * ------------------------- SEARCH DIALOGS ---------------------------- *
   * ===================================================================== */

  /** Method searchKGS()
   * <p> This method will open the Search PC Dialog to allow the user to
   *     retrieve Digital LAS File & Formation Tops Comma Delimited File
   *     from their PC.
   */

  private void searchKGS()
  {
    if (pKGS != null)
      pKGS.close();
    pKGS = null;

    pKGS = new kgsSearchKGSFrame( stStruct, notifier );
  }

  /* ===================================================================== *
   * --------------------------- LOAD DATA ------------------------------- *
   * ===================================================================== */

  /** Method loadData()
   * <p> This method will load the data
   */

  private void loadData()
  {
    if (plot != null)
      plot.close();
    plot = null;

    plot = new lasPlotFrame(
            iAzimuth,
            iCalipers,
            degrees,
		    iqstrat.iqstratControlUtility.copy( stControl ),
			iqstrat.iqstratHeadersUtility.transfer( stHeader ),
			// LAS Data
            las.lasFileDataUtility.transfer( getLAS() ));

    clear();
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnKGS1) { searchKGS(); }

    if (event.getSource() == btnPC1)
    {
      if (p3 != null)
        p3.delete();
      p3 = null;

      p3 = new las3Read( notifier, stStruct );
      p3.getPath();
    }

    if (event.getSource() == btnContinue) { loadData(); } // Load Data
    if (event.getSource() == btnClear)    { clear(); }    // Clear Data

    setButtons();

    if (event.getSource() == btnExit)     { close(); }    // Exit
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String             sArg    = new String( (String) arg);

    if (sArg.equals("Event Selected"))
    {
    }

    /* ------------------------------------------------------------------ *
     * --------------------- LAS LAS File Actions ----------------------- *
     * ------------------------------------------------------------------ */

    if (sArg.equals("KGS LAS File Loaded"))
    {
      getData( _KGS, _HEADERS );
      getData( _KGS, _LAS );

      if (iLog > 0)
      {
     	ARRAY_TEXT[_LAS_R][_NO_C]  = new String( EMPTY );
     	ARRAY_TEXT[_LAS_R][_KGS_C] = new String( YES );

     	if (iLog == 1) { txtLAS1.setText(stFiles.stItem[0].sFilename); }
     	if (iLog == 2) { txtLAS2.setText(stFiles.stItem[1].sFilename); }
     	if (iLog == 3) { txtLAS3.setText(stFiles.stItem[2].sFilename); }
      }
    }

    /* ================================================================== *
     * ------------------- LAS Version 3 Read Actions ------------------- *
     * ================================================================== */

    if (sArg.equals("LAS 3 - Loading Data"))
    {
      iWait = 1;
	  lblWait.setText( "*** Please Wait -- Loading LAS 3.0 File Data ***" );
	}

    if (sArg.equals("LAS 3 - Curves Selected"))
    {
      double d3 = 2.0;

      if (p3 != null)
      {
        getData( _LAS_3_FILE, _HEADERS );      // Header Information
        getData( _LAS_3_FILE, _LAS );          // Log Data

        // Set the Plot Controls

        stControl = iqstrat.iqstratControlUtility.copy( p3.getControlData() );
        if (stControl == null) stControl = new iqstratControlStruct();
        stControl = iqstrat.iqstratControlUtility.addHeaders(
                      stHeader, stControl );

        // Set Table

        if (iLog > 0)
        {
		  if (cmn.cmnString.isNumeric(stLog[iLog-1].sVersion))
		  {
		    d3 = cmn.cmnString.stringToDouble(stLog[iLog-1].sVersion);
		  }
		}

        if (iLog > 0)
        {
       	  ARRAY_TEXT[_LAS_R][_NO_C]   = new String( EMPTY );

          if (d3 == 3.0)
      	    ARRAY_TEXT[_LAS_R][_LAS_3C] = new String( YES );
          else
      	    ARRAY_TEXT[_LAS_R][_LAS_C]  = new String( YES );

          if (iLog == 1) { txtLAS1.setText(stFiles.stItem[0].sFilename); }
      	  if (iLog == 2) { txtLAS2.setText(stFiles.stItem[1].sFilename); }
     	  if (iLog == 3) { txtLAS3.setText(stFiles.stItem[2].sFilename); }
        }

        iWait = 0;

        p3.delete();
        p3 = null;
      }
    }

	setTable();
	setButtons();
  }
}

/** CLASS lasStartFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class lasStartFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 06/22/2016
 *  @author  John Victorine
 */
