/*
 * @las3CurvesFrame.java Version 1.1 01/27/2010
 *
 * Copyright (c) 2010 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package las3.gui;

import java.io.*;
import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;                     // Global Common Data Structure

import iqstrat.iqstratStruct;             // Global IQSTRAT Data Structure
import iqstrat.iqstratControlStruct;      // Plot Control Data Structure

import las.lasFileDataStruct;             // LAS File Data Structure

import las3.las3ListStruct;               // LAS 3 Data List Structure
import las3.las3Struct;                   // LAS 3 Data Structure
import las3.gui.las3CurvesLogPanel;       // LAS 3 Curves Log Panel

/** Class las3CurvesFrame
 *  <p> This Class will display the Curves sections of the LAS 3 File and allow
 *      the user to select the curves they wish to display on the KGS web apps
 *      and to identify unknown curves with KGS Standard Curves.
 *
 *  @version 1.1 01/27/2010
 *  @author  John R. Victorine
 */

public class las3CurvesFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private Observable         pNotifier = null; // Top Observables Object
  private iqstratStruct      stStruct  = null; // Global IQSTRAT Data Structure
  private cmnStruct          stCMN     = null; // Common Data Structure
  private las3ListStruct     stLAS3    = null; // LAS 3 File Data List Structure
  private lasFileDataStruct  stLAS     = null; // LAS File Data Structure

  // Required LAS 3.0 Data Sections default is all data.

  private boolean            bLog[]    = {true,true,true,true,true,
                                          true,true,true,true,true,
                                          true,true,true,true,true,
                                          true,true,true,true,true,
                                          true};

  // Global Variables

  private Observable              notifier    = null; // Observables Object
  private int                     iqstrato    = -1;

  private String                  sReadMethod = "";  // File Read Method
  private String                  sDirectory  = "";  // Directory Path
  private String                  sFilename   = "";  // File name of LAS File

  private int                     iLogs       = 0;
  private lasFileDataStruct       stLogs[]    = null; // LAS File Data Structure

  private int                     iControl    = 0;
  private iqstratControlStruct    stControl[] = null; // Control Data Structure

  private int                     iTotal      = 0;    // Total Number of Panels
  private las3CurvesLogPanel      pCurves[]   = null; // Curve Selection Panel
  private String                  sCurves[]   = null;
  private int                     iData[][]   = null; // LAS 3 Order, Type, Array

  // Use either step depth
  public static final int _SAME   = 0; // Both LAS Files have same step value
  // One Step Depth is greater than the other use the greater value as step depth
  public static final int _STEP_1 = 1; // Step value for 1 is less than 2
  public static final int _STEP_2 = 2; // Step value for 2 is less than 1
  // Both have 1 ft depth step, but one starts at 1/2 foot and the other
  // starts at a 1 foot depth value select the second file for interpolation.
  public static final int _SAME_I = 3; // Both LAS Files have same step value
  // Both have .5 or less step size, but one starts at a different depth than
  // the other need to normalize the depth
  public static final int _SAME_S = 4; // Both LAS Files have same step value

  private int    iStep  = _SAME;

  public static final int _NONE          = las3.las3Constants._NONE;

  public static final int _VERSION       = las3.las3Constants._VERSION;
  public static final int _WELL          = las3.las3Constants._WELL;
  public static final int _LOG           = las3.las3Constants._LOG;
  public static final int _CORE          = las3.las3Constants._CORE;
  public static final int _DRILLING      = las3.las3Constants._DRILLING;
  public static final int _INCLINOMETRY  = las3.las3Constants._INCLINOMETRY;
  public static final int _TOPS          = las3.las3Constants._TOPS;
  public static final int _TEST          = las3.las3Constants._TEST;
  public static final int _PERFORATION   = las3.las3Constants._PERFORATION;
  public static final int _IQ_CONTROL    = las3.las3Constants._IQ_CONTROL;
  public static final int _IQ_LOG        = las3.las3Constants._IQ_LOG;
  public static final int _IQ_CORE       = las3.las3Constants._IQ_CORE;
  public static final int _IQ_SEQ_STRAT  = las3.las3Constants._IQ_SEQ_STRAT;
  public static final int _IQ_FLOW       = las3.las3Constants._IQ_FLOW;
  public static final int _IQ_PFEFFER    = las3.las3Constants._IQ_PFEFFER;
  public static final int _IQ_GEO_REPORT = las3.las3Constants._IQ_GEO_REPORT;
  public static final int _IQ_SHALE      = las3.las3Constants._IQ_SHALE;
  public static final int _IQ_IMAGES     = las3.las3Constants._IQ_IMAGES;
  public static final int _IQ_BRINE      = las3.las3Constants._IQ_BRINE;
  public static final int _IQ_BIO        = las3.las3Constants._IQ_BIO;
  public static final int _IQ_ENV        = las3.las3Constants._IQ_ENV;

  public static final int _LAS_TYPE      = las3.gui.las3ToolsTable._LAS_TYPE;
  public static final int _CORE_TYPE     = las3.gui.las3ToolsTable._CORE_TYPE;
  public static final int _TOPS_TYPE     = las3.gui.las3ToolsTable._TOPS_TYPE;
  public static final int _PERF_TYPE     = las3.gui.las3ToolsTable._PERF_TYPE;
  public static final int _CNTRL_TYPE    = las3.gui.las3ToolsTable._CNTRL_TYPE;
  public static final int _SEQ_TYPE      = las3.gui.las3ToolsTable._SEQ_TYPE;
  public static final int _FLOW_TYPE     = las3.gui.las3ToolsTable._FLOW_TYPE;
  public static final int _GEO_TYPE      = las3.gui.las3ToolsTable._GEO_TYPE;
  public static final int _SHALE_TYPE    = las3.gui.las3ToolsTable._SHALE_TYPE;
  public static final int _IMAGES_TYPE   = las3.gui.las3ToolsTable._IMAGES_TYPE;
  public static final int _BRINE_TYPE    = las3.gui.las3ToolsTable._BRINE_TYPE;
  public static final int _BIO_TYPE      = las3.gui.las3ToolsTable._BIO_TYPE;
  public static final int _ENV_TYPE      = las3.gui.las3ToolsTable._ENV_TYPE;
  public static final int _DST_TYPE      = las3.gui.las3ToolsTable._DST_TYPE;

  // Global Frame Widgets

  private JButton btnContinue = new JButton();

  /** Constructor las3CurvesFrame()
   *  <p> This is the Constructor for this class.
   *  @param pNotifier = Observable
   *  @param stStruct  = Global IQSTRAT Data Structure
   *  @param stLAS     = LAS File Data Structure
   *  @param stLAS3    = LAS 3 File Data List Structure
   *  @param bLog      = Indicator to identify which Data Sections to
   *                     make available ( required data sections )
   */

  public las3CurvesFrame( Observable        pNotifier,
                          iqstratStruct     stStruct,
                          lasFileDataStruct stLAS,
                          las3ListStruct    stLAS3,
                          boolean           bLog[] )
  {
    try
    {
      this.pNotifier = pNotifier;
      this.stStruct  = stStruct;
      this.stCMN     = stStruct.stCMN;
      this.stLAS     = stLAS;
      this.stLAS3    = stLAS3;
      this.bLog      = bLog;

      if (stLAS != null)
      {
        sReadMethod = new String( stLAS.sReadMethod );  // File Read Method
        sDirectory  = new String( stLAS.sDirectory );   // Directory Path
        sFilename   = new String( stLAS.sFilename );    // File name of LAS File
      }

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    int i=0;
    int j=0;
    int iCount = 0;

    JPanel      pnlBase    = new JPanel();
    JTabbedPane tabbedPane = new JTabbedPane();

    // Create an Observable

    notifier = new las3CurvesFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame

    this.setTitle("LAS File Curve Sections");
    this.getContentPane().setLayout(new BorderLayout());

    // Base Panel

    pnlBase.setBorder(BorderFactory.createEtchedBorder());

    btnContinue.setFont(new java.awt.Font("Dialog", 1, 11));
    btnContinue.setText("Continue");
    btnContinue.addActionListener(this);

    // Center Panel

    tabbedPane.setFont(new java.awt.Font("Dialog", 1, 11));

    // Add Log Panel

    if (stLAS3 != null)
    {
      for (i=0; i<stLAS3.iCount; i++)
      {
        switch (stLAS3.stItem[i].iType)
        {
          case _NONE:
          case _VERSION:       // Version Informaiton
          case _WELL:          // Well Information
            break;
          case _LOG:           // Log Data Section
            if (bLog[stLAS3.stItem[i].iType])
            {
              iLogs++;
              iTotal++;
		    }
            break;
          case _CORE:          // Core Data Section
          case _TOPS:          // Tops Data Section
          case _PERFORATION:   // Perforation Data Section
          case _DRILLING:      // Drilling Data Section
          case _INCLINOMETRY:  // Inclinometry Data Section
          case _TEST:          // Test Data Section
            break;
          case _IQ_CONTROL:
            if (bLog[stLAS3.stItem[i].iType])
            {
              iControl++;
              iTotal++;
		    }
            break;
          case _IQ_BIO:        // Bio-Stratigraphy Data Section
          case _IQ_ENV:        // Depositional Environment Data Section
          case _IQ_SEQ_STRAT:
          case _IQ_FLOW:
          case _IQ_PFEFFER:
          case _IQ_LOG:
          case _IQ_CORE:
          case _IQ_GEO_REPORT:
          case _IQ_SHALE:
          case _IQ_IMAGES:
          case _IQ_BRINE:
            break;
        }
      }

      if (iLogs > 0)
      {
        stLogs = new lasFileDataStruct[iLogs];
        for (i=0; i<iLogs; i++)
          stLogs[i] = new lasFileDataStruct();
      }

      if (iControl > 0)
      {
        stControl = new iqstratControlStruct[iControl];
        for (i=0; i<iControl; i++)
          stControl[i] = new iqstratControlStruct();
      }

      if (iTotal > 0)
      {
        pCurves = new las3CurvesLogPanel[iTotal];
        sCurves = new String[iTotal];
        iData   = new int[iTotal][3];   // LAS 3 Order, Type, Array

        for (i=0; i<iTotal; i++)
        {
          sCurves[i] = new String("");
          for (j=0; j<3; j++)
            iData[i][j] = -1;
        }

        for (i=0; i<stLAS3.iCount; i++)
        {
          iqstrato = -1;
          switch (stLAS3.stItem[i].iType)
          {
            case _LOG:         // Log Data Section
            case _CORE:        // Core Data Section
            case _PERFORATION: // Perforation Data Section
              break;
            case _TOPS:        // Tops Data Section
              break;
          }

          switch (stLAS3.stItem[i].iType)
          {
            case _LOG:           // Log Data Section
            case _CORE:          // Core Data Section
            case _TOPS:          // Tops Data Section
            case _TEST:          // TEST Data Section
            case _PERFORATION:   // Perforation Data Section
            case _IQ_CONTROL:    // PROFILE - Control Plot Data
            case _IQ_SEQ_STRAT:  // PROFILE - Sequence Stratigraphy
            case _IQ_FLOW:       // PROFILE - PfEFFER Data
            case _IQ_GEO_REPORT: // PROFILE - Remarks/Comments/Notes
            case _IQ_SHALE:      // PROFILE - Shale Gamma Ray Limits
            case _IQ_IMAGES:     // PROFILE - Rock Images
            case _IQ_BRINE:      // PROFILE - Brine Data
            case _IQ_BIO:        // PROFILE - Bio Stratigraphy Data
            case _IQ_ENV:        // PROFILE - Depositional Environment Data
              if (bLog[stLAS3.stItem[i].iType])
              {
                if (iCount < iTotal)
                {
                  pCurves[iCount] = new las3CurvesLogPanel(
                      notifier, stStruct, stLAS,
                      stLAS3.dVersion, iqstrato, stLAS3.stItem[i] );

                  if (pCurves[iCount] != null)
                  {
                    sCurves[iCount] = new String( pCurves[iCount].getLabel() );
                    iData[iCount][0] = i;
                    iData[iCount][1] = pCurves[iCount].getTypeID();
                    iData[iCount][2] = pCurves[iCount].getArray();
                  }

                  tabbedPane.add(pCurves[iCount],  sCurves[iCount]);
                  iCount++;
				}
              }
              break;
          }
        }
      }
    }

    // Attach Widgets to Frame

    this.getContentPane().add(pnlBase, BorderLayout.SOUTH);
    pnlBase.add(btnContinue, null);

    this.getContentPane().add(tabbedPane, BorderLayout.CENTER);

    // Display the Frame

    this.setSize(new Dimension(750, 550));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width - this.getSize().width) / 2,
                     (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getLAS3()
   * <p> This method will retrieve the LAS & Core Curve Selection and modify
   *     the LAS 3 File Data List Sturcture to retain the users selection and
   *     redefinition of the curve mnemonics to the KGS mnemonics
   * @return stList = modified LAS 3 File Data List Sturcture
   */

  public las3ListStruct getLAS3()
  {
    las3ListStruct stList      = null;
    las3Struct     st          = null;
    int            iSelected[] = null;

    stList = las3.las3Utility.copyList( stLAS3 );

    for (int i=0; i<iTotal; i++)
    {
      if (pCurves[i] != null) // Curve Selection Panel
      {
        if (pCurves[i].getDataType() == _LAS_TYPE)
        {
          st     = las3.las3Utility.copy( pCurves[i].getLAS3() );
          stList = las3.las3Utility.modify( st, stList );
        }

        if (pCurves[i].getDataType() == _CORE_TYPE)
        {
          st     = las3.las3Utility.copy( pCurves[i].getLAS3() );
          stList = las3.las3Utility.modify( st, stList );
        }

        st.delete();
      }
    }

    return (stList);
  }

  /** Method getLAS()
   * <p> This method will retrieve the LAS File Data Structure
   * @return stLAS = the LAS File Data Structure
   */

  public lasFileDataStruct getLAS()
  {
    int               i      = 0;
    int               iCount = 0;
    int               iRows  = 0;
    lasFileDataStruct st     = null;
    double            dNull  = 0.0;
    int               iTemp  = 0;
    double            dStep  = 0.0;
    double            dLogMin = 0.0;
    double            dLogMax = 0.0;

    iTemp            = (int) stLAS.depthStart;
    stLAS.depthStart = (double) iTemp;

    iTemp            = 1 + (int) stLAS.depthEnd;
    stLAS.depthEnd   = (double) iTemp;

    dStep = 0.5;
    if (stLAS.depthStep < 0.5)
      dStep = 0.25;
    if (stLAS.depthStep < 0.25)
      dStep = 0.1;

    stLAS.depthStep  = dStep;

    iRows = 1+(int)(Math.abs(stLAS.depthEnd-stLAS.depthStart)/stLAS.depthStep);

    for (i=0; i<iTotal; i++)
    {
      if (pCurves[i] != null) // Curve Selection Panel
      {
        if (iCount < iLogs)
        {
          if ((pCurves[i].getDataType() == _LAS_TYPE) &&
              (pCurves[i].getAction() == cmn.cmnStruct.ON))
          {
            if (stLAS.iLogs == 0)
            {
              stLAS.sReadMethod = new String( sReadMethod ); // File Read Method
              stLAS.sDirectory  = new String( sDirectory );  // Directory Path
              stLAS.sFilename   = new String( sFilename );   // File name of LAS File

              stLAS.stLAS3      = new las3Struct[iLogs];
              stLAS.iLogs       = iLogs;

              stLAS = las3.las3LoadLogData.initData( iRows, stLAS );
            }

            stLAS = pCurves[i].getLAS( stLAS3.dVersion, iCount, stLAS );

            stLogs[iCount] = las.lasFileDataUtility.copyHeader( stLAS, stLogs[iCount] );
            stLogs[iCount] = pCurves[i].getLASData(
                 stLAS3.sDelim, stLAS3.sWrap, stLAS3.dVersion, iCount, stLogs[iCount] );
            stLAS          = las3.las3LoadLogData.mergeLogData( stLogs[iCount], stLAS );

            iCount++;
          }
        }
      }
    }

    if (iCount > 0)
    {
      if (stLogs != null)
      {
        for (i=0; i<iCount; i++)
        {
          if (stLogs[i] != null)
          {
            stLogs[i].delete();
            stLogs[i] = null;
          }
        }
        stLogs = null;
      }
    }
/*
    if (stLAS != null)
    {
	  if (stLAS.iSP > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dSP, stLAS.dNull);
        stLAS.dSPMin = dRange[0];
        stLAS.dSPMax = dRange[1];
	  }

	  if (stLAS.iLIN_1 > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dLIN_1, stLAS.dNull);
        stLAS.dLIN_1Min = dRange[0];
        stLAS.dLIN_1Max = dRange[1];
	  }

	  if (stLAS.iLIN_2 > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dLIN_2, stLAS.dNull);
        stLAS.dLIN_2Min = dRange[0];
        stLAS.dLIN_2Max = dRange[1];
	  }

	  if (stLAS.iLIN_3 > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dLIN_3, stLAS.dNull);
        stLAS.dLIN_3Min = dRange[0];
        stLAS.dLIN_3Max = dRange[1];
	  }

	  if (stLAS.iLIN_4 > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dLIN_4, stLAS.dNull);
        stLAS.dLIN_4Min = dRange[0];
        stLAS.dLIN_4Max = dRange[1];
	  }

	  if (stLAS.iLOG_1 > -1)
	  {
        double dRange[] = math.mathLAS.computeLogMinMax(stLAS.dLOG_1, stLAS.dNull);
        stLAS.dLOG_1Min = dRange[0];
        stLAS.dLOG_1Max = dRange[1];
	  }

	  if (stLAS.iLOG_2 > -1)
	  {
        double dRange[] = math.mathLAS.computeLogMinMax(stLAS.dLOG_2, stLAS.dNull);
        stLAS.dLOG_2Min = dRange[0];
        stLAS.dLOG_2Max = dRange[1];
	  }

	  if (stLAS.iLOG_3 > -1)
	  {
        double dRange[] = math.mathLAS.computeLogMinMax(stLAS.dLOG_3, stLAS.dNull);
        stLAS.dLOG_3Min = dRange[0];
        stLAS.dLOG_3Max = dRange[1];
	  }

	  if (stLAS.iLOG_4 > -1)
	  {
        double dRange[] = math.mathLAS.computeLogMinMax(stLAS.dLOG_4, stLAS.dNull);
        stLAS.dLOG_4Min = dRange[0];
        stLAS.dLOG_4Max = dRange[1];
	  }

	  if ((stLAS.iLOG_1 > -1) || (stLAS.iLOG_2 > -1) ||
	      (stLAS.iLOG_3 > -1) || (stLAS.iLOG_4 > -1))
	  {
		dLogMin = 0.1;
		dLogMax = 1000.0;

		if (stLAS.iLOG_1 > -1)
		{
		  if (stLAS.dLOG_1Min < dLogMin)
		  {
		    dLogMin = stLAS.dLOG_1Min;
		  }

		  if (stLAS.dLOG_1Max > dLogMax)
		  {
		    dLogMax = stLAS.dLOG_1Max;
		  }
		}

		if (stLAS.iLOG_2 > -1)
		{
		  if (stLAS.dLOG_2Min < dLogMin)
		  {
		    dLogMin = stLAS.dLOG_2Min;
		  }

		  if (stLAS.dLOG_2Max > dLogMax)
		  {
		    dLogMax = stLAS.dLOG_2Max;
		  }
		}

		if (stLAS.iLOG_3 > -1)
		{
		  if (stLAS.dLOG_3Min < dLogMin)
		  {
		    dLogMin = stLAS.dLOG_3Min;
		  }

		  if (stLAS.dLOG_3Max > dLogMax)
		  {
		    dLogMax = stLAS.dLOG_3Max;
		  }
		}

		if (stLAS.iLOG_4 > -1)
		{
		  if (stLAS.dLOG_4Min < dLogMin)
		  {
		    dLogMin = stLAS.dLOG_4Min;
		  }

		  if (stLAS.dLOG_4Max > dLogMax)
		  {
		    dLogMax = stLAS.dLOG_4Max;
		  }
		}

        stLAS.dLOG_1Min = dLogMin;
        stLAS.dLOG_1Max = dLogMax;


        stLAS.dLOG_2Min = dLogMin;
        stLAS.dLOG_2Max = dLogMax;

        stLAS.dLOG_3Min = dLogMin;
        stLAS.dLOG_3Max = dLogMax;

        stLAS.dLOG_4Min = dLogMin;
        stLAS.dLOG_4Max = dLogMax;
	  }
	}
*/

    return (stLAS);
  }

  /** Method getControl()
   * <p> This method will retrieve the Control Data Structure
   * @return st = the Control Data Structure
   */

  public iqstratControlStruct getControl()
  {
    int i        = 0;
    int iCount   = 0;
    int iSamples = 0;
    iqstratControlStruct stR = null;

    for (i=0; i<iTotal; i++)
    {
      if (pCurves[i] != null) // Curve Selection Panel
      {
        if (pCurves[i].getDataType() == _CNTRL_TYPE)

        {
          if (iCount < iControl)
          {
		    if (pCurves[i].getAction() == cmn.cmnStruct.ON)
		    {
              stControl[iCount] =
                iqstrat.iqstratControlUtility.copy( pCurves[i].getControl() );
			}
            iCount++;
          }
        }
      }
    }

    if (stControl != null)
    {
      if (stControl[0] != null)
      {
        stR = iqstrat.iqstratControlUtility.copy( stControl[0] );
        stControl[0].delete();
        stControl[0] = null;
      }
      stControl = null;
    }

    return (stR);
  }

  /* ===================================================================== *
   * ---------------------------- COMPUTE -------------------------------- *
   * ===================================================================== */

  /** Method computeDepths()
   * <p> This method will compute the initial depth information for the
   *     new LAS File
   * @param  iCount = total number of LAS File Data Structures
   * @return st     = LAS File Data Structure
   */

  private lasFileDataStruct computeDepths( int iCount )
  {
    lasFileDataStruct st = new lasFileDataStruct();
    int     i,j;
    double  dTemp      = 0.0;
    int     iSteps     = 0;
    int     iStart[]   = null;
    double  dStart[]   = null;
    double  dStep[]    = null;
    int     iStepVal[] = null;
    // Allowed Step Size  1.0, 0.5, 0.1, other
    int     iValue[]   = { 0,   0,   0,  0 };

    int     iDepth     = 0;
    double  diff       = 0.0;

    if (iCount > 0)
    {
      iStart   = new int[iCount];
      dStart   = new double[iCount];
      dStep    = new double[iCount];
      iStepVal = new int[iCount];

      for (i=0; i<iCount; i++)
      {
        iStart[i]   = 0;
        dStart[i]   = 0.0;
        dStep[i]    = 0.0;
        iStepVal[i] = _SAME;

        if (stLogs[i].depthStart > stLogs[i].depthEnd)
        {
          dTemp                = stLogs[i].depthStart;
          stLogs[i].depthStart = stLogs[i].depthEnd;
          stLogs[i].depthEnd   = dTemp;

          stLogs[i].depthStep  = Math.abs(stLogs[i].depthStep);
        }

        dStart[i] = stLogs[i].depthStart;
        iStart[i] = (int) dStart[i];
        dStart[i] = dStart[i] - (double) iStart[i];
        dStep[i]  = stLogs[i].depthStep;

        if (dStep[i] == 1.0)      iValue[0]++;
        else if (dStep[i] == 0.5) iValue[1]++;
        else if (dStep[i] == 0.1) iValue[2]++;
        else                      iValue[3]++;

        if (i==0)
        {
          st.depthStart = stLogs[0].depthStart;
          st.depthEnd   = stLogs[0].depthEnd;
        }

        dTemp  = stLogs[i].depthStart;
        if (dTemp < st.depthStart) st.depthStart = dTemp;

        dTemp  = stLogs[i].depthEnd;
        if (dTemp > st.depthEnd) st.depthEnd = dTemp;
      }
    }

    st.depthStep = dStep[0];

    if (iValue[3] > 0)
    {
      for (i=0; i<iCount; i++)
      {
        if (dStep[i] < 1.0) st.depthStep = 1.0;
        if (dStep[i] < 0.5) st.depthStep = 0.5;
        if (dStep[i] < 0.1) st.depthStep = 0.1;
      }
    }

    if ((iValue[0] > 0) && (iValue[1] > 0))
      st.depthStep = 1.0;

    if ((iValue[0] > 0) && (iValue[2] > 0))
      st.depthStep = 1.0;

    if ((iValue[1] > 0) && (iValue[2] > 0))
      st.depthStep = 0.5;

    iDepth = (int) st.depthStart * 10;
    diff   = (st.depthStart * 10.0 - (double) iDepth) / 10.0;

    if (st.depthStart < 0.0) st.depthStart = 0.0;
    else if (diff != 0.0)    st.depthStart = (double) iDepth;

    st.iRows = 1 + (int)( Math.abs(st.depthEnd-st.depthStart) / st.depthStep );

    return (st);
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    int i=0;

    // Input Variables

    pNotifier = null; // Top Observables Object
    stStruct  = null; // Global IQSTRAT Data Structure
    stCMN     = null; // Common Data Structure
    stLAS3    = null; // LAS 3 File Data List Structure
    stLAS     = null; // LAS File Data Structure

    if (stLogs != null)
    {
      for (i=0; i<iLogs; i++)
      {
        if (stLogs[i] != null)
          stLogs[i].delete();
        stLogs[i] = null;
      }
    }
    stLogs = null;
    iLogs  = 0;

    // Global Variables

    sReadMethod = null;  // File Read Method
    sDirectory  = null;  // Directory Path
    sFilename   = null;  // File name of LAS File

    if (stControl != null)
    {
      for (i=0; i<iControl; i++)
      {
        if (stControl[i] != null)
          stControl[i].delete();
        stControl[i] = null;
      }
    }
    stControl = null;
    iControl  = 0;

    notifier = null;   // Observables Object

    for (i=0; i<iTotal; i++)
      pCurves[i] = null;   // Curve Selection Panel

    sCurves = null;
    iData   = null;   // LAS 3 Order, Type, Array
    iTotal  = 0;      // Total Number of Panels

    // Global Frame Widgets

    btnContinue = null;

    dispose();
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnContinue)
    {
      if (pNotifier != null)
        pNotifier.notifyObservers(new String("LAS 3 - Curves Selected"));
    }
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("Get LAS Standard Tool"))
    {
      for (int i=0; i<iTotal; i++)
      {
        if (pCurves[i] != null) pCurves[i].getCurve();
      }
    }
  }
}

/** CLASS las3CurvesFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class las3CurvesFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 01/27/2010
 *  @author  John Victorine
 */
