/*
 * @plotXYStruct.java Version 1.1 12/16/2004
 *
 * Copyright (c) 2004 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import plot.plotSymbolStruct;

/** Class plotXYStruct
 *  <p> This class will hold the data necessary to create a Plot.
 *
 *  @version 1.1 12/16/2004
 *  @author  John R. Victorine
 *  modified 2 April 2004
 */

public class plotXYStruct
{
  public static final int LINEAR = 0;
  public static final int LOG    = 1;

  public int          iWidth  = 0;      // Width of the Plot Area
  public int          iHeight = 0;     // Height of the Plot Area

  public int          iWo     = 0;     // Width of the 2D Plot Area
  public int          iHo     = 0;     // Height of the 2D Plot Area

  // Sampling of Data

  public String       sampling = "Log Sampled Every 10'";

  // Titles

  public String       sTitle1 = "";      // The First Line of the Title
  public String       sTitle2 = "";      // The Second Line of the Title
  public String       sTitle3 = "";      // The Third Line of the Title

  // Step Size

  public double       dStep   = 24.0;

  // X-Axis Values.

  public int          iXLog       = 0;   // X-Axis Semilog Indicator
                                         //   1-Semilog
                                         //   0-Linear
  public int          iXCycles    = 1;   // Number of Cycles
  public double       dXMaximum   = 0.0; // Maximum X-Axis
  public double       dXMinimum   = 0.0; // Minimum X-Axis
  public double       dXIncrement = 0.0; // Increment along the X-Axis
  public String       sXMnemonic  = "";  // X-Axis Mnemonic
  public String       sXAxis      = "";  // X-Axis Label.

  // Y-Axis Values.

  public int          iYLog       = 0;   // Y-Axis Semilog Indicator
                                         //   1-Semilog
                                         //   0-Linear
  public int          iYCycles    = 1;   // Number of Cycles
  public double       dYMaximum   = 0.0; // Maximum Y-Axis
  public double       dYMinimum   = 0.0; // Minimum Y-Axis
  public double       dYIncrement = 0.0; // Increment along the Y-Axis
  public String       sYMnemonic  = "";  // Y-Axis Mnemonic
  public String       sYAxis      = "";  // Y-Axis Label.

  // Z-Axis Values.

  public int          iZLog       = 0;   // Z-Axis Semilog Indicator
                                         //   1-Semilog
                                         //   0-Linear
  public int          iZCycles    = 1;   // Number of Cycles
  public double       dZMaximum   = 0.0; // Maximum Z-Axis
  public double       dZMinimum   = 0.0; // Minimum Z-Axis
  public double       dZIncrement = 0.0; // Increment along the Z-Axis
  public String       sZMnemonic  = "";  // Z-Axis Mnemonic
  public String       sZAxis      = "";  // Z-Axis Label.

  // Plot Data.

  public int              iRows       = 0;    // Number of Rows
  public int              iBins       = 0;    // Number of Bins
  public double           depths[]    = null; // Array of Depths
  public double           dX[]        = null; // Array of Horizontal Axis Data
  public double           dY[]        = null; // Array of Vertical Axis Data
  public double           dZ[]        = null; // Array of Other Data
  public String           sZ[]        = null; // Data descriptor array for filtering
  public plotSymbolStruct stSymbols[] = null; // Array of Symbol & colors

  // Legend

  public String           sLegRange  = "";    // Depth Range
  public String           sLegTitle  = "";    // Legend Title
  public boolean          bLegend    = false; // Legend OFF
  public int              iLegend    = 0;     // Number of Legends
  public plotSymbolStruct stLegend[] = null;  // Array of Symbols & colors

  /** Method delete()
   *  <p> This method will set the arrays and strings to null to force Java to
   *     deallocate memory
   */

  public void delete()
  {
    iWidth      = 0;    // Width of the Plot Area
    iHeight     = 0;    // Height of the Plot Area

    // Sampling of Data

    sampling = null;

    // Titles

    sTitle1     = null; // The First Line of the Title
    sTitle2     = null; // The Second Line of the Title
    sTitle3     = null; // The Third Line of the Title

    // X-Axis Values.

    iXLog       = 0;    // X-Axis Semilog Indicator
                        //   1-Semilog
                        //   0-Linear
    iXCycles    = 1;    // Number of Cycles
    dXMaximum   = 0.0;  // Maximum X-Axis
    dXMinimum   = 0.0;  // Minimum X-Axis
    dXIncrement = 0.0;  // Increment along the X-Axis
    sXMnemonic  = null; // X-Axis Mnemonic
    sXAxis      = null; // X-Axis Label.

    // Y-Axis Values.

    iYLog       = 0;    // Y-Axis Semilog Indicator
                        //   1-Semilog
                        //   0-Linear
    iYCycles    = 1;    // Number of Cycles
    dYMaximum   = 0.0;  // Maximum Y-Axis
    dYMinimum   = 0.0;  // Minimum Y-Axis
    dYIncrement = 0.0;  // Increment along the Y-Axis
    sYMnemonic  = null; // Y-Axis Mnemonic
    sYAxis      = null; // Y-Axis Label.

    // Z-Axis Values.

    iZLog       = 0;    // Z-Axis Semilog Indicator
                        //   1-Semilog
                        //   0-Linear
    iZCycles    = 1;    // Number of Cycles
    dZMaximum   = 0.0;  // Maximum Z-Axis
    dZMinimum   = 0.0;  // Minimum Z-Axis
    dZIncrement = 0.0;  // Increment along the Z-Axis
    sZMnemonic  = null; // Z-Axis Mnemonic
    sZAxis      = null; // Z-Axis Label.

    // Plot Data.

    iRows       = 0;    // Number of Rows
    iBins       = 0;    // Number of Bins
    depths      = null; // Array of Depths
    dX          = null; // Array of Horizontal Axis Data
    dY          = null; // Array of Vertical Axis Data
    dZ          = null; // Array of Other Data
    sZ          = null; // Data descriptor array for filtering
    stSymbols   = null; // Array of Symbol & colors

    // Legend

    sLegRange   = null;  // Depth Range
    sLegTitle   = null;  // Legend Title
    bLegend     = false; // Legend Off
    iLegend     = 0;     // Number of Legends
    stLegend    = null;  // Array of Symbols & colors
  }
}